

#include "cyu3system.h"
#include "cyu3os.h"
#include "cyu3dma.h"
#include "cyu3error.h"
#include "usb3vision.h"
#include "cyu3usb.h"
#include "cyu3uart.h"
#include "cyu3gpio.h"
#include "cyu3utils.h"
#include "cyu3pib.h"
#include "cyfxgpif2config.h"

 CyU3PThread Usb3VisionAppThread;

 CyU3PEvent glUsb3VisionEvent;

 CyU3PDmaChannel glChHandleDSI, glChHandleDCICmd, glChHandleDCIRsp, glChHandleDEI;

 uint32_t *uniqueIDreg = (uint32_t *)0xE0055010;
 uint32_t uniqueIDL, uniqueIDH;

 CyBool_t glIsApplnActive = 0;
 CyBool_t header = 0;
 uint32_t blockcount = 0;
 uint8_t i = 0;
 uint32_t conscount = 0;

 CyFxU3VCCD CCD;
 uint32_t SCD_regaddr = 0;
 uint32_t SCD_length = 0;
 uint32_t acqstart = 0;
 uint32_t acqstop = 0;

 extern uint8_t CyU3VXML[];


/* Application Error Handler */
void
CyFxAppErrorHandler (
        CyU3PReturnStatus_t apiRetStatus    /* API return status */
        )
{
    /* Application failed with the error code apiRetStatus */

    /* Add custom debug or recovery actions here */

    /* Loop Indefinitely */
    for (;;)
    {
        /* Thread sleep : 100 ms */
    	CyU3PDebugPrint(1,"\r\n App Error Handler");
        CyU3PThreadSleep (1000);
    }
}


uint8_t CyU3VABRM[528] =
{
	0x00, 0x00, 0x01, 0x00,			// GenCP Version: Minor version (16bits) = 0x0000 , Major version (16bits) = 0x0001
	U3V_MANUFACTURER_NAME,			// Manufacturer Name String in ASCII, 64 bytes
	U3V_MODEL_NAME,					// Model Name String in ASCII, 64 bytes
	U3V_FAMILY_NAME,				// Family Name string in ASCII, 64 bytes
	U3V_DEVICE_VERSION,				// Device Version String in ASCII, 64 bytes
	NULL_64,						// Manufacturer info string, 64 bytes, NULL
	NULL_64,						// Serial Number string, 64 bytes, to be filled during runtime by CyFxU3VUpdateSerialNum()
	NULL_64,						// User-defined name string, 64 bytes, NULL
	U3V_DEVICE_CAPABILITIES,		// Device Capabilities (SBRM, Family name reg supported)
	0x2C, 0x01, 0x00, 0x00,			// Max. device response time = 300ms (spec limit)
	U3V_MANIFEST_TABLE_ADDR,		// Manifest table base address
	U3V_SBRM_ADDR,					// SBRM base address
	NULL_8,							// Device Configuration, heartbeat not supported
	0x00, 0x00, 0x00, 0x00,			// Heartbeat timeout, not supported
	0x00, 0x00, 0x00, 0x00,			// Message channel ID, not supported
	NULL_8,							// Timestamp, not supported
	0x00, 0x00, 0x00, 0x00,			// Timestamp latch, not supported
	NULL_8,							// Timestamp increment, not supported
	0x00, 0x00, 0x00, 0x00,			// Access Privilege, available
	0x00, 0x00, 0x00, 0x00,			// Protocol endianness, not supported
	0x00, 0x00, 0x00, 0x00			// Implementation endianness, not supported
};

uint8_t CyU3VSBRM[68] =
{
	0x00, 0x00, 0x01, 0x00,			// U3V Version: Minor version (16bits) = 0x0000 , Major version (16bits) = 0x0001
	U3V_CAPABILITIES,				// U3V Capabilities: SIRM supported
	NULL_8,							// U3V Configuration, reserved, set to 0
	0x00, 0x04, 0x00, 0x00,			// Max. Command transfer length = 1024
	0x00, 0x04, 0x00, 0x00,			// Max. Ack. transfer length = 1024
	0x01, 0x00, 0x00, 0x00,			// No. of stream channels = 1
	U3V_SIRM_ADDR,					// SIRM base address
	0xFF, 0x00, 0x00, 0x00,			// SIRM length
	NULL_8,							// EIRM base addr, not supported
	0x00, 0x00, 0x00, 0x00,			// EIRM length, not supported
	NULL_8,							// IIDC2 base address, not supported
	0x08, 0x00, 0x00, 0x00,			// Current Speed, SuperSpeed by default
};

uint8_t CyU3VSIRM[48] =
{
	0x00, 0x00, 0x00, 0x00,			// SI info: 1 byte alignment
	0x00, 0x00, 0x00, 0x00,			// SI control: stream not enabled by default
	0x00, 0x10, 0x0E, 0x00,			// SI required payload size: frame size = 1280*720
	0x00, 0x00, 0x00, 0x00,			// upper 4 bytes of required payload size
	0x34, 0x00, 0x00, 0x00,			// SI required leader size: 52 bytes, image leader
	0x20, 0x00, 0x00, 0x00,			// SI required trailer size: 32 bytes, image trailer
	0x34, 0x00, 0x00, 0x00,			// SI maximum leader size: 52 bytes
	0x00, 0x3C, 0x00, 0x00,			// SI payload transfer size: 15k (DMA buffer size)
	0x3C, 0x00, 0x00, 0x00,			// SI payload transfer count: 60 full (15k) buffers in one frame
	0x00, 0x00, 0x00, 0x00,			// SI payload final transfer1 size: 0, not needed, no partial buffer
	0x00, 0x00, 0x00, 0x00,			// SI payload final transfer2 size: 0, not needed, no partial buffers
	0x20, 0x00, 0x00, 0x00			// SI maximum trailer size: 32 bytes
};


uint8_t CyU3VManifest[] =
{
	0x01, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,			// No. of Manifest entries = 1

	0x00, 0x00, 0x00, 0x01,			// GenICam file version 1.0.0
	0x00, 0x00, 0x01, 0x01,			// Schema/Filetype Uncompressed xml ver. 1.1.0
	U3V_XML_ADDR,					// Address for XML file
	0xFF, 0xFF, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,			// Size of xml file (8 bytes width)
	NULL_20,						// SHA-1 for xml - not available
	NULL_20							// Reserved
};

uint8_t CyU3VXMLFeatures[] =
{
	0x00, 0x05,	0x00, 0x00,			// Width of frame = 1280 pixels per line
	0xD0, 0x02,	0x00, 0x00,			// Height of frame = 720 lines per frame
	0x01, 0x00, 0x08, 0x01,			// Pixel format = 0x01080001, Mono8 format
	0x00, 0x10, 0x0E, 0x00,			// Payload size = 0xE1000 (1280*720)
	0x01, 0x00, 0x00, 0x00,			// Acquisition Mode = 1 (Continuous)
	0x00, 0x00, 0x00, 0x00,			// Acquisition Start
	0x00, 0x00, 0x00, 0x00			// Acquisition Stop
};


uint8_t CyU3VLeader[52] =
{
	0x4C, 0x56, 0x33, 0x55,			// Magic Key 0x4C563355 'U3VL'
	0x00, 0x00,						// reserved, set to 0
	0x34, 0x00,						// Leader size = 52 bytes
	NULL_8,							// Block ID, starts with 0
	0x00, 0x00,						// reserved, set to 0
	0x01, 0x00,						// Payload type: Image
	NULL_8,							// Timestamp, always 0, not supported
	0x01, 0x00, 0x08, 0x01,			// Pixel format = 0x01080001, Mono8 format
	0x00, 0x05,	0x00, 0x00,			// Width of frame = 1280 pixels per line
	0xD0, 0x02,	0x00, 0x00,			// Height of frame = 720 lines per frame
	0x00, 0x00, 0x00, 0x00,			// Width_Offset = 0, no ROI support
	0x00, 0x00,	0x00, 0x00,			// Height_Offset = 0, no ROI support
	0x00, 0x00,						// Horizontal padding = 0, not supported
	0x00, 0x00						// reserved, set to 0
};

uint8_t CyU3VTrailer[32] =
{
	0x54, 0x56, 0x33, 0x55,			// Magic key 0x54563355 'U3VT'
	0x00, 0x00,						// reserved, set to 0
	0x20, 0x00,						// Trailer size = 32 bytes
	NULL_8,							// Block ID, starts with 0
	0x00, 0x00,						// Status
	0x00, 0x00,						// reserved, set to 0
	U3V_VALID_PAYLOAD_SIZE,			// Valid payload size = 1280*720 bytes
	0xD0, 0x02,	0x00, 0x00			// Actual height of frame = 720 lines
};


void CyFxUSB3VisionApplnDebugInit (void)
{
    CyU3PGpioClock_t  gpioClock;
    CyU3PUartConfig_t uartConfig;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Initialize the GPIO block. If we are transitioning from the boot app, we can verify whether the GPIO
       state is retained. */
    gpioClock.fastClkDiv = 2;
    gpioClock.slowClkDiv = 32;
    gpioClock.simpleDiv  = CY_U3P_GPIO_SIMPLE_DIV_BY_16;
    gpioClock.clkSrc     = CY_U3P_SYS_CLK_BY_2;
    gpioClock.halfDiv    = 0;
    apiRetStatus = CyU3PGpioInit (&gpioClock, NULL);

    /* Initialize the UART for printing debug messages */
    apiRetStatus = CyU3PUartInit();
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        /* Error handling */
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Set UART configuration */
    CyU3PMemSet ((uint8_t *)&uartConfig, 0, sizeof (uartConfig));
    uartConfig.baudRate = CY_U3P_UART_BAUDRATE_115200;
    uartConfig.stopBit = CY_U3P_UART_ONE_STOP_BIT;
    uartConfig.parity = CY_U3P_UART_NO_PARITY;
    uartConfig.txEnable = CyTrue;
    uartConfig.rxEnable = CyFalse;
    uartConfig.flowCtrl = CyFalse;
    uartConfig.isDma = CyTrue;

    apiRetStatus = CyU3PUartSetConfig (&uartConfig, NULL);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Set the UART transfer to a really large value. */
    apiRetStatus = CyU3PUartTxSetBlockXfer (0xFFFFFFFF);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Initialize the debug module. */
    apiRetStatus = CyU3PDebugInit (CY_U3P_LPP_SOCKET_UART_CONS, 8);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(apiRetStatus);
    }

    CyU3PDebugPreamble(CyFalse);
}


void CyFxU3VUpdateBlockCount (void)
{
	if (blockcount == 0)
	{
		blockcount++;
		return;
	}

	for(i=0; i<4; i++)
	{
		CyU3VLeader[8+i] = ((blockcount>>8*i) & 0xFF);
		CyU3VTrailer[8+i] = ((blockcount>>8*i) & 0xFF);
	}

	blockcount++;
}


void CyFxUSB3VisionDSIDmaCallback( CyU3PDmaChannel *handle,        /**< Handle to the DMA channel. */
        CyU3PDmaCbType_t type,          /**< The type of callback notification being generated. */
        CyU3PDmaCBInput_t *input  )
{
	CyU3PReturnStatus_t stat = CY_U3P_SUCCESS;

	if(type == CY_U3P_DMA_CB_PROD_EVENT)
	{
		if(input->buffer_p.count == 0)
		{
			if(header)
			{
				CyFxU3VUpdateBlockCount();
				CyU3PMemCopy(input->buffer_p.buffer, CyU3VLeader, 52);

				stat = CyU3PDmaChannelCommitBuffer(handle, 52, 0);
				if(stat != CY_U3P_SUCCESS)
				{
					CyU3PDebugPrint(1,"\r\n CyU3PDmaChannelCommitBuffer failed for header %x", stat);
					CyFxAppErrorHandler(stat);
				}
				header = 0;
			}
			else
			{
				CyU3PMemCopy(input->buffer_p.buffer, CyU3VTrailer, 32);

				stat = CyU3PDmaChannelCommitBuffer(handle, 32, 0);
				if(stat != CY_U3P_SUCCESS)
				{
					CyU3PDebugPrint(1,"\r\n CyU3PDmaChannelCommitBuffer failed for trailer %x", stat);
					CyFxAppErrorHandler(stat);
				}
				header = 1;
			}
		}

		else
		{
			stat = CyU3PDmaChannelCommitBuffer(handle, input->buffer_p.count, 0);
			if(stat != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(1,"\r\n CyU3PDmaChannelCommitBuffer failed %x", stat);
				CyFxAppErrorHandler(stat);
			}
		}
	}

	else if (type  == CY_U3P_DMA_CB_CONS_EVENT)
	{
		conscount++;

		if (conscount == 62)			// line size = 1280 bytes; total frame size = 921600 bytes; total buffers for frame = 921600/15360 = 60 + leader + trailer => 62 buffers in one frame
		{
			conscount = 0;

			if(!acqstop)
			{
				stat = CyU3PGpifSMSwitch(257, 0, 257, 0, 2);
				if(stat != CY_U3P_SUCCESS)
				{
					CyU3PDebugPrint(1,"\r\n CyU3PGpifSMSwitch failed, error = %x",stat);
					CyFxAppErrorHandler(stat);
				}
			}
		}
	}
}



void CyFxUSB3VisionApplnStart (void)
{
    uint16_t size = 0;
    CyU3PEpConfig_t epCfg;
    CyU3PDmaChannelConfig_t dmaCfg;
    CyU3PReturnStatus_t stat = CY_U3P_SUCCESS;
    CyU3PUSBSpeed_t usbSpeed = CyU3PUsbGetSpeed();

    /* First identify the usb speed. Once that is identified,
     * create a DMA channel and start the transfer on this. */

    /* Based on the Bus Speed configure the endpoint packet size */
    switch (usbSpeed)
    {
    case CY_U3P_FULL_SPEED:
        size = 64;
        break;

    case CY_U3P_HIGH_SPEED:
        size = 512;
        break;

    case  CY_U3P_SUPER_SPEED:
        size = 1024;
        break;

    default:
        CyU3PDebugPrint (4, "Error! Invalid USB speed.\n");
        CyFxAppErrorHandler (CY_U3P_ERROR_FAILURE);
        break;
    }

    CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
    epCfg.enable = CyTrue;
    epCfg.epType = CY_U3P_USB_EP_BULK;
    epCfg.burstLen = 1;
    epCfg.streams = 0;
    epCfg.pcktSize = size;

    /* DCI Out EP configuration */
    stat = CyU3PSetEpConfig(CY_FX_EP_DCI_CMD, &epCfg);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PSetEpConfig failed for CY_FX_EP_DCI_CMD, Error code = %x\n", stat);
        CyFxAppErrorHandler (stat);
    }

    /* DCI In EP configuration */
    stat = CyU3PSetEpConfig(CY_FX_EP_DCI_RSP, &epCfg);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PSetEpConfig failed for CY_FX_EP_DCI_RSP, Error code = %x\n", stat);
        CyFxAppErrorHandler (stat);
    }

    /* DEI In EP configuration */
//    stat = CyU3PSetEpConfig(CY_FX_EP_DEI_EVENT, &epCfg);
//    if (stat != CY_U3P_SUCCESS)
//    {
//        CyU3PDebugPrint (4, "CyU3PSetEpConfig failed for CY_FX_EP_DEI_EVENT, Error code = %x\n", stat);
//        CyFxAppErrorHandler (stat);
//    }

    epCfg.burstLen = (usbSpeed == CY_U3P_SUPER_SPEED) ? CY_FX_STREAM_EP_BURST : 1;

    /* DSI In EP configuration */
    stat = CyU3PSetEpConfig(CY_FX_EP_DSI_STREAM, &epCfg);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PSetEpConfig failed for CY_FX_EP_DSI_STREAM, Error code = %x\n", stat);
        CyFxAppErrorHandler (stat);
    }

    /* Flush the endpoint memory */
    CyU3PUsbFlushEp(CY_FX_EP_DCI_CMD);
    CyU3PUsbFlushEp(CY_FX_EP_DCI_RSP);
//    CyU3PUsbFlushEp(CY_FX_EP_DEI_EVENT);
    CyU3PUsbFlushEp(CY_FX_EP_DSI_STREAM);

    /* Create a DMA MANUAL_IN channel for the producer socket. */
    CyU3PMemSet ((uint8_t *)&dmaCfg, 0, sizeof (dmaCfg));

    dmaCfg.size  = (size * CY_FX_STREAM_EP_BURST);
    dmaCfg.count = CY_FX_USB3VISION_DMA_BUF_COUNT;
    dmaCfg.prodSckId = CY_U3P_PIB_SOCKET_0;
    dmaCfg.consSckId = (CY_U3P_UIB_SOCKET_CONS_0 | (CY_FX_EP_DSI_STREAM & 0x0F));
    dmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
    dmaCfg.notification = CY_U3P_DMA_CB_PROD_EVENT | CY_U3P_DMA_CB_CONS_EVENT;
    dmaCfg.cb = CyFxUSB3VisionDSIDmaCallback;
    dmaCfg.prodHeader = 0;
    dmaCfg.prodFooter = 0;
    dmaCfg.consHeader = 0;
    dmaCfg.prodAvailCount = 0;

    stat = CyU3PDmaChannelCreate (&glChHandleDSI,
            CY_U3P_DMA_TYPE_MANUAL, &dmaCfg);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PDmaChannelCreate failed for glChHandleDSI, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* Create a DMA MANUAL_OUT channel for the consumer socket. */
    dmaCfg.size = size;
    dmaCfg.count = 1;
    dmaCfg.notification = NULL;
    dmaCfg.cb = NULL;
    dmaCfg.prodSckId = (CY_U3P_UIB_SOCKET_PROD_0 | (CY_FX_EP_DCI_CMD & 0x0F));
    dmaCfg.consSckId = CY_U3P_CPU_SOCKET_CONS;
    stat = CyU3PDmaChannelCreate (&glChHandleDCICmd,
            CY_U3P_DMA_TYPE_MANUAL_IN, &dmaCfg);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PDmaChannelCreate failed for glChHandleDCICmd, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    dmaCfg.prodSckId = CY_U3P_CPU_SOCKET_PROD;
    dmaCfg.consSckId = (CY_U3P_UIB_SOCKET_CONS_0 | (CY_FX_EP_DCI_RSP & 0x0F));
    stat = CyU3PDmaChannelCreate (&glChHandleDCIRsp,
            CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaCfg);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PDmaChannelCreate failed for glChHandleDCIRsp, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

//    dmaCfg.count = 1;
//    dmaCfg.notification = NULL;
//    dmaCfg.cb = NULL;
//    dmaCfg.prodSckId = CY_U3P_CPU_SOCKET_PROD;
//    dmaCfg.consSckId = (CY_U3P_UIB_SOCKET_CONS_0 | (CY_FX_EP_DEI_EVENT & 0x0F));
//    stat = CyU3PDmaChannelCreate (&glChHandleDEI,
//            CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaCfg);
//    if (stat != CY_U3P_SUCCESS)
//    {
//        CyU3PDebugPrint (4, "CyU3PDmaChannelCreate failed for glChHandleDEI, Error code = %x\n", stat);
//        CyFxAppErrorHandler(stat);
//    }

//    /* Set DMA Channel transfer size */
//    stat = CyU3PDmaChannelSetXfer (&glChHandleDSI, 0);
//    if (stat != CY_U3P_SUCCESS)
//    {
//        CyU3PDebugPrint (4, "CyU3PDmaChannelSetXfer failed for glChHandleDSI, Error code = %x\n", stat);
//        CyFxAppErrorHandler(stat);
//    }

    /* Set DMA Channel transfer size */
    stat = CyU3PDmaChannelSetXfer (&glChHandleDCICmd, 0);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PDmaChannelSetXfer failed for glChHandleDCICmd, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* Set DMA Channel transfer size */
    stat = CyU3PDmaChannelSetXfer (&glChHandleDCIRsp, 0);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PDmaChannelSetXfer failed for glChHandleDCIRsp, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* Set DMA Channel transfer size */
//    stat = CyU3PDmaChannelSetXfer (&glChHandleDEI, 0);
//    if (stat != CY_U3P_SUCCESS)
//    {
//        CyU3PDebugPrint (4, "CyU3PDmaChannelSetXfer failed for glChHandleDEI, Error code = %x\n", stat);
//        CyFxAppErrorHandler(stat);
//    }

    /* Update the flag so that the application thread is notified of this. */
    glIsApplnActive = CyTrue;
}



void CyFxUSB3VisionApplnStop (void)
{
	CyU3PEpConfig_t epcfg;
	CyU3PReturnStatus_t stat = CY_U3P_SUCCESS;

	glIsApplnActive = CyFalse;

	CyU3PDebugPrint(1,"\r\n Appln Stop initiated");

	CyU3PDmaChannelDestroy(&glChHandleDCICmd);
	CyU3PDmaChannelDestroy(&glChHandleDCIRsp);
	CyU3PDmaChannelDestroy(&glChHandleDSI);

	CyU3PUsbFlushEp(CY_FX_EP_DCI_CMD);
	CyU3PUsbFlushEp(CY_FX_EP_DCI_RSP);
	CyU3PUsbFlushEp(CY_FX_EP_DSI_STREAM);

	CyU3PMemSet(&epcfg, 0x00, sizeof(epcfg));
	epcfg.enable = CyFalse;

	stat = CyU3PSetEpConfig(CY_FX_EP_DCI_CMD, &epcfg);
	if(stat != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(1,"\r\n CyU3PSetEpConfig failed DCI_CMD error=%x", stat);
		CyFxAppErrorHandler(stat);
	}

	stat = CyU3PSetEpConfig(CY_FX_EP_DCI_RSP, &epcfg);
	if(stat != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(1,"\r\n CyU3PSetEpConfig failed DCI_RSP error=%x", stat);
		CyFxAppErrorHandler(stat);
	}

	stat = CyU3PSetEpConfig(CY_FX_EP_DSI_STREAM, &epcfg);
	if(stat != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(1,"\r\n CyU3PSetEpConfig failed DSI_STREAM error=%x", stat);
		CyFxAppErrorHandler(stat);
	}
}


CyBool_t CyFxUSB3VisionApplnUSBSetupCB (uint32_t setupdat0,  uint32_t setupdat1)
{
    /* Fast enumeration is used. Only requests addressed to the interface, class,
     * vendor and unknown control requests are received by this function.
     * This application does not support any class or vendor requests. */

    uint8_t  bRequest, bReqType;
    uint8_t  bType, bTarget;
    uint16_t wValue, wIndex, wLength;
    CyBool_t isHandled = CyFalse;

    /* Decode the fields from the setup request. */
    bReqType = (setupdat0 & CY_U3P_USB_REQUEST_TYPE_MASK);
    bType    = (bReqType & CY_U3P_USB_TYPE_MASK);
    bTarget  = (bReqType & CY_U3P_USB_TARGET_MASK);
    bRequest = ((setupdat0 & CY_U3P_USB_REQUEST_MASK) >> CY_U3P_USB_REQUEST_POS);
    wValue   = ((setupdat0 & CY_U3P_USB_VALUE_MASK)   >> CY_U3P_USB_VALUE_POS);
    wIndex   = ((setupdat1 & CY_U3P_USB_INDEX_MASK)   >> CY_U3P_USB_INDEX_POS);
    wLength  = ((setupdat1 & CY_U3P_USB_LENGTH_MASK)  >> CY_U3P_USB_LENGTH_POS);

    if (bType == CY_U3P_USB_STANDARD_RQT)
    {

    }

    if ((bType == CY_U3P_USB_VENDOR_RQT) && (bTarget == CY_U3P_USB_TARGET_DEVICE))
    {

    }

    return isHandled;

}


void CyFxUSB3VisionApplnUSBEventCB (CyU3PUsbEventType_t evtype, uint16_t evdata)
{
    switch (evtype)
    {
    	case CY_U3P_USB_EVENT_SETCONF:
    		if(glIsApplnActive)
    		{
    			CyFxUSB3VisionApplnStop();
    		}
    		CyFxUSB3VisionApplnStart();
    		break;

    	default:
    		break;
    }

}



void CyFxUSB3VisionApplnInit (void)
{
	CyU3PReturnStatus_t stat = CY_U3P_SUCCESS;
	CyU3PPibClock_t pibclk;

	pibclk.clkSrc = CY_U3P_SYS_CLK;
	pibclk.clkDiv = 2;
	pibclk.isDllEnable = CyFalse;
	pibclk.isHalfDiv = CyFalse;

	stat = CyU3PPibInit(CyTrue, &pibclk);
	if(stat != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(1,"\r\n CyU3PPibInit failed with error code %x", stat);
	}

	stat = CyU3PUsbStart();
	if(stat != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(1,"\r\n CyU3pUsbStart failed with error code %x", stat);
	}

	CyU3PUsbRegisterSetupCallback(CyFxUSB3VisionApplnUSBSetupCB, CyTrue);

	CyU3PUsbRegisterEventCallback(CyFxUSB3VisionApplnUSBEventCB);

    /* Super speed device descriptor. */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_DEVICE_DESCR, NULL, (uint8_t *)CyFxUSB30DeviceDscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set device descriptor failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* High speed device descriptor. */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_DEVICE_DESCR, NULL, (uint8_t *)CyFxUSB20DeviceDscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set device descriptor failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* BOS descriptor */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_BOS_DESCR, NULL, (uint8_t *)CyFxUSBBOSDscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set configuration descriptor failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* Device qualifier descriptor */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_DEVQUAL_DESCR, NULL, (uint8_t *)CyFxUSBDeviceQualDscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set device qualifier descriptor failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* Super speed configuration descriptor */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_CONFIG_DESCR, NULL, (uint8_t *)CyFxUSBSSConfigDscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set Super Speed configuration descriptor failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* High speed configuration descriptor */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_CONFIG_DESCR, NULL, (uint8_t *)CyFxUSBHSConfigDscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB Set High Speed Descriptor failed, Error Code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* Full speed configuration descriptor */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_FS_CONFIG_DESCR, NULL, (uint8_t *)CyFxUSBFSConfigDscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB Set Full Speed Descriptor failed, Error Code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* String descriptor 0 */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 0, (uint8_t *)CyFxU3VStr0Dscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set string descriptor failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* String descriptor 1 */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 1, (uint8_t *)CyFxUSBVendorDscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set string descriptor failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* String descriptor 2 */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 2, (uint8_t *)CyFxUSBProductDscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set string descriptor failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* String descriptor 3 */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 3, (uint8_t *)CyFxUSBFunctionDscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set string descriptor failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    /* String descriptor 4 */
    stat = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 4, (uint8_t *)CyFxU3VGUIDDscr);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set string descriptor failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    CyU3PDebugPrint (4, "About to connect to USB host\r\n");

    /* Connect the USB Pins with super speed operation enabled. */
        stat = CyU3PConnectState(CyTrue, CyTrue);
        if (stat != CY_U3P_SUCCESS)
        {
            CyU3PDebugPrint (4, "USB Connect failed, Error code = %x\n", stat);
            CyFxAppErrorHandler(stat);
        }

        CyU3PDebugPrint(1, "\r\n CyFxUSB3VisionApplnInit complete");
}


void CyFxU3VGpifStart (void)
{
	CyU3PReturnStatus_t stat = CY_U3P_SUCCESS;

    /* Start DSI DMA channel */
    stat = CyU3PDmaChannelSetXfer (&glChHandleDSI, 0);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PDmaChannelSetXfer failed for glChHandleDSI, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    stat = CyU3PGpifLoad(&CyFxGpifConfig);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PGpifLoad failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }

    stat = CyU3PGpifSMStart(START, ALPHA_START);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PGpifSMStart failed, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }
}


void CyFxU3VGpifStop (void)
{
	CyU3PReturnStatus_t stat = CY_U3P_SUCCESS;

	CyU3PGpifDisable(CyTrue);

	stat = CyU3PDmaChannelReset(&glChHandleDSI);
    if (stat != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PDmaChannelReset failed for glChHandleDSI, Error code = %x\n", stat);
        CyFxAppErrorHandler(stat);
    }
}


void CyFxProcessU3VCmd (uint8_t *cmdbuf)
{
//	uint8_t i=0;
//
//	CyU3PDebugPrint(1,"\r\n");
//	for(i=0; i<12; i++)
//	{
//		CyU3PDebugPrint(1," %x ",*(cmdbuf+i));
//	}

	CCD.prefix     = (*(cmdbuf+3)<<24 | *(cmdbuf+2)<<16 | *(cmdbuf+1)<<8 | *(cmdbuf));
	CCD.flags      = (*(cmdbuf+5)<<8 | *(cmdbuf+4));
	CCD.command_id = (*(cmdbuf+7)<<8 | *(cmdbuf+6));
	CCD.length     = (*(cmdbuf+9)<<8 | *(cmdbuf+8));
	CCD.request_id = (*(cmdbuf+11)<<8 | *(cmdbuf+10));

	if(CCD.prefix != U3V_CCD_PREFIX)
	{
		CyU3PDebugPrint(1,"\r\n Invalid prefix = %x", CCD.prefix);
		CCD.flags = GENCP_INVALID_PARAMETER;
		CCD.length = 0;
		*(cmdbuf)   = (U3V_CCD_PREFIX & 0x000000FF);
		*(cmdbuf+1) = (U3V_CCD_PREFIX & 0x0000FF00)>>8;
		*(cmdbuf+2) = (U3V_CCD_PREFIX & 0x00FF0000)>>16;
		*(cmdbuf+3) = (U3V_CCD_PREFIX & 0xFF000000)>>24;
		goto end_func;
	}

	if(CCD.flags & 0x8000)
	{
		CyU3PDebugPrint(1,"\r\n Invalid flag CMD_RESEND received in CCD");
		CCD.flags = U3V_STATUS_RESEND_NOT_SUPPORTED;
		CCD.length = 0;
		goto end_func;
	}

	switch (CCD.command_id)
	{
		case GENCP_READMEM_CMD:
			SCD_regaddr = (*(cmdbuf+15)<<24 | *(cmdbuf+14)<<16 | *(cmdbuf+13)<<8 | *(cmdbuf+12));
			SCD_length  = (*(cmdbuf+23)<<8 | *(cmdbuf+22));
			CyU3PDebugPrint(1,"\r\n READMEM_CMD regaddr=%x length=%x", SCD_regaddr, SCD_length);
			CCD.flags = GENCP_SUCCESS;
			CCD.command_id = GENCP_READMEM_ACK;
			CCD.length = SCD_length;

			switch((SCD_regaddr & 0xFF000000)>>24)
			{
				case 0x00:
					CyU3PMemCopy((cmdbuf+12), (CyU3VABRM+(SCD_regaddr&0xFFFF)), SCD_length);
					break;

				case 0x01:
					CyU3PMemCopy((cmdbuf+12), (CyU3VSBRM+(SCD_regaddr&0xFFFF)), SCD_length);
					break;

				case 0x02:
					CyU3PMemCopy((cmdbuf+12), (CyU3VSIRM+(SCD_regaddr&0xFFFF)), SCD_length);
					break;

				case 0x03:
					CyU3PMemCopy((cmdbuf+12), (CyU3VManifest+(SCD_regaddr&0xFFFF)), SCD_length);
					break;

				case 0x04:
					CyU3PMemCopy((cmdbuf+12), (CyU3VXML+(SCD_regaddr&0xFFFF)), SCD_length);
					break;

				case 0x05:
					CyU3PMemCopy((cmdbuf+12), (CyU3VXMLFeatures+(SCD_regaddr&0xFFFF)), SCD_length);
					break;

				default:
					CyU3PDebugPrint(1,"\r\n Invalid regaddr");
					CCD.flags = GENCP_INVALID_ADDRESS;
					CCD.length = 0;
					SCD_length = 0;
					break;
			}
			break;

		case GENCP_WRITEMEM_CMD:
			SCD_regaddr = (*(cmdbuf+15)<<24 | *(cmdbuf+14)<<16 | *(cmdbuf+13)<<8 | *(cmdbuf+12));
			CyU3PDebugPrint(1,"\r\n WRITEMEM_CMD regaddr=%x", SCD_regaddr);
			CCD.flags = GENCP_SUCCESS;
			CCD.command_id = GENCP_WRITEMEM_ACK;
			CCD.length = 0;

			switch((SCD_regaddr & 0xFF000000)>>24)
			{
				case 0x00:
				case 0x01:
					CCD.flags = GENCP_WRITE_PROTECT;
					break;

				case 0x02:
					CyU3PMemCopy((CyU3VSIRM+(SCD_regaddr&0xFFFF)), (cmdbuf+20), CCD.length);
					break;

				case 0x03:
				case 0x04:
					CCD.flags = GENCP_WRITE_PROTECT;
					break;

				case 0x05:
					if((SCD_regaddr & 0xFFFF) == 0x0014)
					{
						acqstart = (*(cmdbuf+23)<<24 | *(cmdbuf+22)<<16 | *(cmdbuf+21)<<8 | *(cmdbuf+20));
						if(acqstart)
						{
							CyFxU3VGpifStart();
							CyU3PDebugPrint(1,"\r\n GPIFII started");
							acqstop = 0;
							CyU3PMemCopy((CyU3VXMLFeatures+(0x0018)), 0x00, 4);
						}
					}
					else if((SCD_regaddr & 0xFFFF) == 0x0018)
					{
						acqstop = (*(cmdbuf+23)<<24 | *(cmdbuf+22)<<16 | *(cmdbuf+21)<<8 | *(cmdbuf+20));
						if(acqstop)
						{
							CyFxU3VGpifStop();
							CyU3PDebugPrint(1,"\r\n GPIFII Stopped");
							acqstart = 0;
							CyU3PMemCopy((CyU3VXMLFeatures+(0x0014)), 0x00, 4);
						}
					}

					CyU3PMemCopy((CyU3VXMLFeatures+(SCD_regaddr&0xFFFF)), (cmdbuf+20), CCD.length);
					break;

				default:
					CyU3PDebugPrint(1,"\r\n Invalid regaddr");
					CCD.flags = GENCP_INVALID_ADDRESS;
					break;
			}
			break;

		default:
			CyU3PDebugPrint(1,"\r\n Invalid command_id %x", CCD.command_id);
			CCD.flags = GENCP_INVALID_PARAMETER;
			CCD.length = 0;
			break;
	}

	end_func:
	*(cmdbuf+4) = (CCD.flags & 0x00FF);
	*(cmdbuf+5) = (CCD.flags & 0xFF00)>>8;
	*(cmdbuf+6) = (CCD.command_id & 0x00FF);
	*(cmdbuf+7) = (CCD.command_id & 0xFF00)>>8;
	*(cmdbuf+8) = (CCD.length & 0x00FF);
	*(cmdbuf+9) = (CCD.length & 0xFF00)>>8;
}


void CyFxU3VUpdateSerialNum(void)
{
	uint8_t j=0;
	uniqueIDL = *(uniqueIDreg);
	uniqueIDH = *(uniqueIDreg+32);

	for(j=0; j<4; j++)
	{
		CyU3VABRM[260+j] = ((uniqueIDL>>(j*8))&0x000000FF);
		CyU3VABRM[264+j] = ((uniqueIDH>>(j*8))&0x000000FF);
	}
}


void USB3VisionAppThread_Entry (uint32_t input)
{
	CyU3PReturnStatus_t stat = CY_U3P_SUCCESS;
	CyU3PDmaBuffer_t InBuf, OutBuf;

	CyFxU3VUpdateSerialNum();

	CyFxUSB3VisionApplnDebugInit();

	CyU3PDebugPrint(1,"\r\n debug init done");

	CyFxUSB3VisionApplnInit();

	for (;;)
	{
		if( glIsApplnActive)
		{
			stat = CyU3PDmaChannelGetBuffer(&glChHandleDCICmd, &InBuf, CYU3P_WAIT_FOREVER);
			if(stat != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(1,"\r\n CyU3PDmaChannelGetBuffer failed for glChHandleDCICmd, error = %x", stat);
				CyFxAppErrorHandler(stat);
			}

			stat = CyU3PDmaChannelGetBuffer(&glChHandleDCIRsp, &OutBuf, CYU3P_NO_WAIT);
			if(stat != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(1,"\r\n CyU3PDmaChannelGetBuffer failed for glChHandleDCIRsp, error = %x", stat);
				CyFxAppErrorHandler(stat);
			}

			CyFxProcessU3VCmd(InBuf.buffer);

			CyU3PMemCopy(OutBuf.buffer, InBuf.buffer, CCD.length);

			stat = CyU3PDmaChannelDiscardBuffer(&glChHandleDCICmd);
			if(stat != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(1,"\r\n CyU3PDmaChannelDiscardBuffer failed error = %x",stat);
				CyFxAppErrorHandler(stat);
			}

			stat = CyU3PDmaChannelCommitBuffer(&glChHandleDCIRsp, (CCD.length+12), 0);
			if(stat != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(1,"\r\n CyU3PDmaChannelCommitBuffer failed for glChHandleDCIRsp, error = %x",stat);
				CyFxAppErrorHandler(stat);
			}
		}

		else
		{
			CyU3PThreadSleep(100);
		}
	}
}



void CyFxApplicationDefine (void)
{
    void *ptr = NULL;
    uint32_t ret = CY_U3P_SUCCESS;

    /* Create an event flag group that will be used for signalling the application thread. */
    ret = CyU3PEventCreate (&glUsb3VisionEvent);
    if (ret != 0)
    {
        /* Loop indefinitely */
        while (1);
    }

    /* Allocate the memory for the threads */
    ptr = CyU3PMemAlloc (CY_FX_USB3VISION_THREAD_STACK);

    /* Create the thread for the application */
    ret = CyU3PThreadCreate (&Usb3VisionAppThread,                /* App thread structure */
                          "21:USB3 Vision",                      /* Thread ID and thread name */
                          USB3VisionAppThread_Entry,              /* App thread entry function */
                          0,                                       /* No input parameter to thread */
                          ptr,                                     /* Pointer to the allocated thread stack */
                          CY_FX_USB3VISION_THREAD_STACK,          /* App thread stack size */
                          CY_FX_USB3VISION_THREAD_PRIORITY,       /* App thread priority */
                          CY_FX_USB3VISION_THREAD_PRIORITY,       /* App thread priority */
                          CYU3P_NO_TIME_SLICE,                     /* No time slice for the application thread */
                          CYU3P_AUTO_START                         /* Start the thread immediately */
                          );

    /* Check the return code */
    if (ret != 0)
    {
        /* Thread Creation failed with the error code retThrdCreate */

        /* Add custom recovery or debug actions here */

        /* Application cannot continue */
        /* Loop indefinitely */
        while(1);
    }
}


int main (void)
{
	CyU3PIoMatrixConfig_t iocfg;
	CyU3PReturnStatus_t stat = CY_U3P_SUCCESS;

	CyU3PSysClockConfig_t clockcfg;

	clockcfg.clkSrc = CY_U3P_SYS_CLK;
	clockcfg.cpuClkDiv = 2;
	clockcfg.dmaClkDiv = 2;
	clockcfg.mmioClkDiv = 2;
	clockcfg.setSysClk400 = CyTrue;
	clockcfg.useStandbyClk = CyFalse;

	stat = CyU3PDeviceInit(&clockcfg);
	if(stat != CY_U3P_SUCCESS)
	{
		goto handle_fatal_error;
	}

	stat = CyU3PDeviceCacheControl(CyTrue, CyFalse, CyFalse);
	if(stat != CY_U3P_SUCCESS)
	{
		goto handle_fatal_error;
	}

	iocfg.isDQ32Bit = CyFalse;
	iocfg.s0Mode = CY_U3P_SPORT_INACTIVE;
	iocfg.s1Mode = CY_U3P_SPORT_INACTIVE;
	iocfg.useI2C = CyFalse;
	iocfg.useSpi = CyFalse;
	iocfg.useUart = CyTrue;
	iocfg.useI2S = CyFalse;
	iocfg.lppMode = CY_U3P_IO_MATRIX_LPP_UART_ONLY;

	iocfg.gpioSimpleEn[0] = 0;
	iocfg.gpioSimpleEn[1] = 0;
	iocfg.gpioComplexEn[0] = 0;
	iocfg.gpioComplexEn[1] = 0;

	stat = CyU3PDeviceConfigureIOMatrix(&iocfg);
	if(stat != CY_U3P_SUCCESS)
	{
		goto handle_fatal_error;
	}

	CyU3PKernelEntry();

	handle_fatal_error:
	while(1);

}
