


#include "usb3vision.h"

/* Standard device descriptor for USB 3.0 */
const uint8_t CyFxUSB30DeviceDscr[] __attribute__ ((aligned (32))) =
{
    0x12,                           /* Descriptor size */
    CY_U3P_USB_DEVICE_DESCR,        /* Device descriptor type */
    0x00,0x03,                      /* USB 3.0 */
    0xEF,                           /* Device class */
    0x02,                           /* Device sub-class */
    0x01,                           /* Device protocol */
    0x09,                           /* Maxpacket size for EP0 : 2^9 */
    0xB4,0x04,                      /* Vendor ID */
    0xBA,0xDC,                      /* Product ID */
    0x00,0x00,                      /* Device release number */
    0x01,                           /* Manufacture string index */
    0x02,                           /* Product string index */
    0x00,                           /* Serial number string index */
    0x01                            /* Number of configurations */
};

/* Standard device descriptor for USB 2.0 */
const uint8_t CyFxUSB20DeviceDscr[] __attribute__ ((aligned (32))) =
{
    0x12,                           /* Descriptor size */
    CY_U3P_USB_DEVICE_DESCR,        /* Device descriptor type */
    0x10,0x02,                      /* USB 2.10 */
    0xEF,                           /* Device class */
    0x02,                           /* Device sub-class */
    0x01,                           /* Device protocol */
    0x40,                           /* Maxpacket size for EP0 : 64 bytes */
    0xB4,0x04,                      /* Vendor ID */
    0xBA,0xDC,                      /* Product ID */
    0x00,0x00,                      /* Device release number */
    0x01,                           /* Manufacture string index */
    0x02,                           /* Product string index */
    0x00,                           /* Serial number string index */
    0x01                            /* Number of configurations */
};

/* Binary device object store descriptor */
const uint8_t CyFxUSBBOSDscr[] __attribute__ ((aligned (32))) =
{
    0x05,                           /* Descriptor size */
    CY_U3P_BOS_DESCR,               /* Device descriptor type */
    0x16,0x00,                      /* Length of this descriptor and all sub descriptors */
    0x02,                           /* Number of device capability descriptors */

    /* USB 2.0 extension */
    0x07,                           /* Descriptor size */
    CY_U3P_DEVICE_CAPB_DESCR,       /* Device capability type descriptor */
    CY_U3P_USB2_EXTN_CAPB_TYPE,     /* USB 2.0 extension capability type */
    0x02,0x00,0x00,0x00,            /* Supported device level features: LPM support  */

    /* SuperSpeed device capability */
    0x0A,                           /* Descriptor size */
    CY_U3P_DEVICE_CAPB_DESCR,       /* Device capability type descriptor */
    CY_U3P_SS_USB_CAPB_TYPE,        /* SuperSpeed device capability type */
    0x00,                           /* Supported device level features  */
    0x0E,0x00,                      /* Speeds supported by the device : SS, HS and FS */
    0x03,                           /* Functionality support */
    0x0A,                           /* U1 Device Exit latency */
    0xFF,0x07                       /* U2 Device Exit latency */
};

/* Standard device qualifier descriptor */
const uint8_t CyFxUSBDeviceQualDscr[] __attribute__ ((aligned (32))) =
{
    0x0A,                           /* Descriptor size */
    CY_U3P_USB_DEVQUAL_DESCR,       /* Device qualifier descriptor type */
    0x00,0x02,                      /* USB 2.0 */
    0xEF,                           /* Device class */
    0x02,                           /* Device sub-class */
    0x01,                           /* Device protocol */
    0x40,                           /* Maxpacket size for EP0 : 64 bytes */
    0x01,                           /* Number of configurations */
    0x00                            /* Reserved */
};

/* Standard super speed configuration descriptor */
const uint8_t CyFxUSBSSConfigDscr[] __attribute__ ((aligned (32))) =
{
    /* Configuration descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_CONFIG_DESCR,        /* Configuration descriptor type */
    0x5E, 0x00,                     /* Length of this descriptor and all sub descriptors */
    0x02,                           /* Number of interfaces */
    0x01,                           /* Configuration number */
    0x00,                           /* COnfiguration string index */
    0x80,                           /* Config characteristics - Bus powered */
    0x32,                           /* Max power consumption of device (in 8mA unit) : 400mA */

    /* Interface Association Descriptor */
    0x08,							/* Descriptor size */
    CY_FX_INTF_ASSN_DSCR_TYPE,		/* IAD type */
    0x00,							/* bFirstInterface */
    0x02,							/* bInterfaceCount */
    0xEF,							/* bFunctionClass */
    0x05,							/* bFunctionSubClass */
    0x00,							/* bFunctionProtocol */
    0x02,							/* Function string index */

    /* Device Control Interface Descriptor */
    0x09,							/* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,		/* Interface Descriptor */
    0x00,							/* bInterfaceNumber */
    0x00,							/* bAlternateSetting */
    0x02,							/* bNumEndpoints */
    0xEF,							/* bInterfaceClass */
    0x05,							/* bInterfaceSubClass */
    0x00,							/* bInterfaceProtocol */
    0x00,							/* Interface String index */

    /* Class-specific Device Info Descriptor */
    0x14,							/* Descriptor Size */
    0x24,							/* Descriptor type = U3V_INTERFACE */
    0x01,							/* Descriptor Subtype = U3V_DEVICEINFO */
    0x00, 0x00, 0x01, 0x00,			/* GenCP Version */
    0x00, 0x00, 0x01, 0x00,			/* U3V Version */
    0x03,							/* U3V GUID String index */
    0x04,							/* Vendor Name String index */
    0x00,							/* Model Name String index */
    0x00,							/* Family Name String index */
    0x00,							/* Device Version String index */
    0x00,							/* Manufacturer info string index */
    0x00,							/* Serial Number string index */
    0x00,							/* User Defined Name string index */
    0x0C,							/* USB Speed Support - SS and HS supported */

    /* Endpoint descriptor */
    0x07,							/* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,		/* Endpoint descriptor type */
    CY_FX_EP_DCI_CMD,				/* bEndpoint Address */
    CY_U3P_USB_EP_BULK,				/* bmAttributes */
    0x00, 0x04,						/* wMaxPacketSize */
    0x00,							/* bInterval */

    /* SS Endpoint Companion Descriptor */
    0x06,							/* Descriptor size */
    CY_U3P_SS_EP_COMPN_DESCR,		/* EP Companion descriptor type */
    0x00,							/* bMaxBurst */
    0x00,							/* bmAttributes */
    0x00, 0x00,						/* wBytesPerInterval */

    /* Endpoint descriptor */
    0x07,							/* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,		/* Endpoint descriptor type */
    CY_FX_EP_DCI_RSP,				/* bEndpoint Address */
    CY_U3P_USB_EP_BULK,				/* bmAttributes */
    0x00, 0x04,						/* wMaxPacketSize */
    0x00,							/* bInterval */

    /* SS Endpoint Companion Descriptor */
    0x06,							/* Descriptor size */
    CY_U3P_SS_EP_COMPN_DESCR,		/* EP Companion descriptor type */
    0x00,							/* bMaxBurst */
    0x00,							/* bmAttributes */
    0x00, 0x00,						/* wBytesPerInterval */

//    /* Device Event Interface Descriptor */
//    0x09,							/* Descriptor size */
//    CY_U3P_USB_INTRFC_DESCR,		/* Interface Descriptor */
//    0x01,							/* bInterfaceNumber */
//    0x00,							/* bAlternateSetting */
//    0x01,							/* bNumEndpoints */
//    0xEF,							/* bInterfaceClass */
//    0x05,							/* bInterfaceSubClass */
//    0x01,							/* bInterfaceProtocol */
//    0x00,							/* Interface String index */
//
//    /* Endpoint descriptor */
//    0x07,							/* Descriptor size */
//    CY_U3P_USB_ENDPNT_DESCR,		/* Endpoint descriptor type */
//    CY_FX_EP_DEI_EVENT,				/* bEndpoint Address */
//    CY_U3P_USB_EP_BULK,				/* bmAttributes */
//    0x00, 0x04,						/* wMaxPacketSize */
//    0x00,							/* bInterval */
//
//    /* SS Endpoint Companion Descriptor */
//    0x06,							/* Descriptor size */
//    CY_U3P_SS_EP_COMPN_DESCR,		/* EP Companion descriptor type */
//    0x00,							/* bMaxBurst */
//    0x00,							/* bmAttributes */
//    0x00, 0x00,						/* wBytesPerInterval */

    /* Device Streaming Interface Descriptor */
    0x09,							/* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,		/* Interface Descriptor */
    0x01,							/* bInterfaceNumber */
    0x00,							/* bAlternateSetting */
    0x01,							/* bNumEndpoints */
    0xEF,							/* bInterfaceClass */
    0x05,							/* bInterfaceSubClass */
    0x02,							/* bInterfaceProtocol */
    0x00,							/* Interface String index */

    /* Endpoint descriptor */
    0x07,							/* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,		/* Endpoint descriptor type */
    CY_FX_EP_DSI_STREAM,			/* bEndpoint Address */
    CY_U3P_USB_EP_BULK,				/* bmAttributes */
    0x00, 0x04,						/* wMaxPacketSize */
    0x00,							/* bInterval */

    /* SS Endpoint Companion Descriptor */
    0x06,							/* Descriptor size */
    CY_U3P_SS_EP_COMPN_DESCR,		/* EP Companion descriptor type */
    (CY_FX_STREAM_EP_BURST-1),		/* bMaxBurst */
    0x00,							/* bmAttributes */
    0x00, 0x00						/* wBytesPerInterval */
};

/* Standard high speed configuration descriptor */
const uint8_t CyFxUSBHSConfigDscr[] __attribute__ ((aligned (32))) =
{
    /* Configuration descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_CONFIG_DESCR,        /* Configuration descriptor type */
    0x4C, 0x00,                     /* Length of this descriptor and all sub descriptors */
    0x02,                           /* Number of interfaces */
    0x01,                           /* Configuration number */
    0x00,                           /* COnfiguration string index */
    0x80,                           /* Config characteristics - Bus powered */
    0x32,                           /* Max power consumption of device (in 8mA unit) : 400mA */

    /* Interface Association Descriptor */
    0x08,							/* Descriptor size */
    CY_FX_INTF_ASSN_DSCR_TYPE,		/* IAD type */
    0x00,							/* bFirstInterface */
    0x02,							/* bInterfaceCount */
    0xEF,							/* bFunctionClass */
    0x05,							/* bFunctionSubClass */
    0x00,							/* bFunctionProtocol */
    0x03,							/* Function string index */

    /* Device Control Interface Descriptor */
    0x09,							/* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,		/* Interface Descriptor */
    0x00,							/* bInterfaceNumber */
    0x00,							/* bAlternateSetting */
    0x02,							/* bNumEndpoints */
    0xEF,							/* bInterfaceClass */
    0x05,							/* bInterfaceSubClass */
    0x00,							/* bInterfaceProtocol */
    0x00,							/* Interface String index */

    /* Endpoint descriptor */
    0x07,							/* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,		/* Endpoint descriptor type */
    CY_FX_EP_DCI_CMD,				/* bEndpoint Address */
    CY_U3P_USB_EP_BULK,				/* bmAttributes */
    0x00, 0x04,						/* wMaxPacketSize */
    0x00,							/* bInterval */

    /* Endpoint descriptor */
    0x07,							/* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,		/* Endpoint descriptor type */
    CY_FX_EP_DCI_RSP,				/* bEndpoint Address */
    CY_U3P_USB_EP_BULK,				/* bmAttributes */
    0x00, 0x04,						/* wMaxPacketSize */
    0x00,							/* bInterval */

    /* Class-specific Device Info Descriptor */
    0x14,							/* Descriptor Size */
    0x24,							/* Descriptor type = U3V_INTERFACE */
    0x01,							/* Descriptor Subtype = U3V_DEVICEINFO */
    0x01, 0x00, 0x00, 0x00,			/* GenCP Version */
    0x01, 0x00, 0x00, 0x00,			/* U3V Version */
    0x04,							/* U3V GUID String index */
    0x01,							/* Vendor Name String index */
    0x00,							/* Model Name String index */
    0x00,							/* Family Name String index */
    0x00,							/* Device Version String index */
    0x00,							/* Manufacturer info string index */
    0x00,							/* Serial Number string index */
    0x00,							/* User Defined Name string index */
    0x0C,							/* USB Speed Support - SS and HS supported */

//    /* Device Event Interface Descriptor */
//    0x09,							/* Descriptor size */
//    CY_U3P_USB_INTRFC_DESCR,		/* Interface Descriptor */
//    0x01,							/* bInterfaceNumber */
//    0x00,							/* bAlternateSetting */
//    0x01,							/* bNumEndpoints */
//    0xEF,							/* bInterfaceClass */
//    0x05,							/* bInterfaceSubClass */
//    0x01,							/* bInterfaceProtocol */
//    0x00,							/* Interface String index */
//
//    /* Endpoint descriptor */
//    0x07,							/* Descriptor size */
//    CY_U3P_USB_ENDPNT_DESCR,		/* Endpoint descriptor type */
//    CY_FX_EP_DEI_EVENT,				/* bEndpoint Address */
//    CY_U3P_USB_EP_BULK,				/* bmAttributes */
//    0x00, 0x04,						/* wMaxPacketSize */
//    0x00,							/* bInterval */

    /* Device Streaming Interface Descriptor */
    0x09,							/* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,		/* Interface Descriptor */
    0x01,							/* bInterfaceNumber */
    0x00,							/* bAlternateSetting */
    0x01,							/* bNumEndpoints */
    0xEF,							/* bInterfaceClass */
    0x05,							/* bInterfaceSubClass */
    0x02,							/* bInterfaceProtocol */
    0x00,							/* Interface String index */

    /* Endpoint descriptor */
    0x07,							/* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,		/* Endpoint descriptor type */
    CY_FX_EP_DSI_STREAM,			/* bEndpoint Address */
    CY_U3P_USB_EP_BULK,				/* bmAttributes */
    0x00, 0x04,						/* wMaxPacketSize */
    0x00,							/* bInterval */
};

/* Standard Full Speed Configuration Descriptor */
const uint8_t CyFxUSBFSConfigDscr[] =
    {

        /* Configuration Descriptor Type */
        0x09,                           /* Descriptor Size */
        CY_U3P_USB_CONFIG_DESCR,        /* Configuration Descriptor Type */
        0x09,0x00,                      /* Length of this descriptor and all sub descriptors */
        0x00,                           /* Number of interfaces */
        0x01,                           /* Configuration number */
        0x00,                           /* COnfiguration string index */
        0x80,                           /* Config characteristics - Bus powered */
        0x32,                           /* Max power consumption of device (in 2mA unit) : 100mA */
    };

/* Standard product string descriptor */
const uint8_t CyFxUSBProductDscr[] __attribute__ ((aligned (32))) =
{
    0x08,                           /* Descriptor size */
    CY_U3P_USB_STRING_DESCR,        /* Device descriptor type */
    'F',0x00,
    'X',0x00,
    '3',0x00
};

/* Function string descriptor */
const uint8_t CyFxUSBFunctionDscr[] __attribute__ ((aligned (32))) =
{
    0x26,                           /* Descriptor size */
    CY_U3P_USB_STRING_DESCR,        /* Device descriptor type */
    'U',0x00,
    'S',0x00,
    'B',0x00,
    '3',0x00,
    ' ',0x00,
    'V',0x00,
    'i',0x00,
    's',0x00,
    'i',0x00,
    'o',0x00,
    'n',0x00,
    ' ',0x00,
    'D',0x00,
    'e',0x00,
    'v',0x00,
    'i',0x00,
    'c',0x00,
    'e',0x00
};

/* U3V GUID string descriptor */
const uint8_t CyFxUSBVendorDscr[] __attribute__ ((aligned (32))) =
{
    0x10,                           /* Descriptor size */
    CY_U3P_USB_STRING_DESCR,        /* Device descriptor type */
    'C',0x00,
    'y',0x00,
    'p',0x00,
    'r',0x00,
    'e',0x00,
    's',0x00,
    's',0x00
};

/* Standard product string descriptor */
const uint8_t CyFxU3VGUIDDscr[] __attribute__ ((aligned (32))) =
{
    0x1A,                           /* Descriptor size */
    CY_U3P_USB_STRING_DESCR,        /* Device descriptor type */
    '0',0x00,
    '4',0x00,
    'B',0x00,
    '4',0x00,
    '4',0x00,
    '3',0x00,
    '2',0x00,
    '1',0x00,
    '5',0x00,
    '6',0x00,
    '7',0x00,
    '8',0x00
};

const uint8_t CyFxU3VStr0Dscr[] __attribute__ ((aligned (32))) =
{
		0x08,
		CY_U3P_USB_STRING_DESCR,
		'D',0x00,
		'E',0x00,
		'F',0x00
};

